@@ -1,5 +1,5 @@
 /*
- *  This file is part of libfaketime, version 0.9.5
+ *  This file is part of libfaketime, version 0.9.6
  *
  *  libfaketime is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License v2 as published by the
@@ -56,6 +56,7 @@
 #define BUFFERLEN   256
 
 #ifndef __APPLE__
+extern char *__progname;
 #include <endian.h>
 #else
 /* endianness related macros */
@@ -83,22 +84,27 @@ typedef int clockid_t;
 #endif
 #endif
 
+/* some systems lack raw clock */
+#ifndef CLOCK_MONOTONIC_RAW
+#define CLOCK_MONOTONIC_RAW (CLOCK_MONOTONIC + 1)
+#endif
+
 /*
  * Per thread variable, which we turn on inside real_* calls to avoid modifying
- * time multiple times
+ * time multiple times of for the whole process to prevent faking time
  */
 __thread bool dont_fake = false;
 
 /* Wrapper for function calls, which we want to return system time */
-#define DONT_FAKE_TIME(call)			\
-  {						                \
-    bool dont_fake_orig = dont_fake;	\
-    if (!dont_fake)                     \
-    {				                    \
-      dont_fake = true;				    \
-    }						            \
-    call;					            \
-    dont_fake = dont_fake_orig;			\
+#define DONT_FAKE_TIME(call)          \
+  {                                   \
+    bool dont_fake_orig = dont_fake;  \
+    if (!dont_fake)                   \
+    {                                 \
+      dont_fake = true;               \
+    }                                 \
+    call;                             \
+    dont_fake = dont_fake_orig;       \
   } while (0)
 
 /* pointers to real (not faked) functions */
@@ -114,10 +120,15 @@ static time_t       (*real_time)            (time_t *);
 static int          (*real_ftime)           (struct timeb *);
 static int          (*real_gettimeofday)    (struct timeval *, void *);
 static int          (*real_clock_gettime)   (clockid_t clk_id, struct timespec *tp);
+#ifdef FAKE_INTERNAL_CALLS
+static int          (*real___ftime)           (struct timeb *);
+static int          (*real___gettimeofday)    (struct timeval *, void *);
+static int          (*real___clock_gettime)   (clockid_t clk_id, struct timespec *tp);
+#endif
 #ifndef __APPLE__
 #ifdef FAKE_TIMERS
 static int          (*real_timer_settime_22)   (int timerid, int flags, const struct itimerspec *new_value,
-			                            	            struct itimerspec * old_value);
+                                                struct itimerspec * old_value);
 static int          (*real_timer_settime_233)  (timer_t timerid, int flags,
                                                 const struct itimerspec *new_value,
                                                 struct itimerspec * old_value);
@@ -160,7 +171,7 @@ struct saved_timestamp
 };
 
 static inline void timespec_from_saved (struct timespec *tp,
-					struct saved_timestamp *saved)
+  struct saved_timestamp *saved)
 {
   /* read as big endian */
   tp->tv_sec = be64toh(saved->sec);
@@ -208,7 +219,7 @@ static bool user_rate_set = false;
 static struct timespec user_per_tick_inc = {0, -1};
 static bool user_per_tick_inc_set = false;
 
-enum ft_mode_t {FT_FREEZE, FT_START_AT} ft_mode = FT_FREEZE;
+enum ft_mode_t {FT_FREEZE, FT_START_AT, FT_NOOP} ft_mode = FT_FREEZE;
 
 /* Time to fake is not provided through FAKETIME env. var. */
 static bool parse_config_file = true;
@@ -245,6 +256,7 @@ static void ft_shm_init (void)
       perror("shm_open");
       exit(1);
     }
+
     if (MAP_FAILED == (ft_shared = mmap(NULL, sizeof(struct ft_shared_s), PROT_READ|PROT_WRITE,
             MAP_SHARED, ticks_shm_fd, 0)))
     {
@@ -353,10 +365,12 @@ static void save_time(struct timespec *tp)
     }
 
     lseek(outfile, 0, SEEK_END);
-    do {
+    do
+    {
       written = write(outfile, &(((char*)&time_write)[n]), sizeof(time_write) - n);
-    } while (((written == -1) && (errno == EINTR)) ||
-             (sizeof(time_write) < (n += written)));
+    }
+    while (((written == -1) && (errno == EINTR)) ||
+            (sizeof(time_write) < (n += written)));
 
     if ((written == -1) || (n < sizeof(time_write)))
     {
@@ -396,14 +410,14 @@ static bool load_time(struct timespec *tp)
 
       if (ft_shared->ticks == 0)
       {
- 	    /* we set shared memory to stop using infile */
-	    ft_shared->ticks = 1;
-    	system_time_from_system(&ftpl_starttime);
-	    ft_shared->start_time = ftpl_starttime;
+        /* we set shared memory to stop using infile */
+        ft_shared->ticks = 1;
+        system_time_from_system(&ftpl_starttime);
+        ft_shared->start_time = ftpl_starttime;
       }
       else
       {
-    	ftpl_starttime = ft_shared->start_time;
+        ftpl_starttime = ft_shared->start_time;
       }
 
       munmap(stss, infile_size);
@@ -446,11 +460,43 @@ static bool load_time(struct timespec *tp)
 
 static int fake_stat_disabled = 0;
 
+#define FAKE_STRUCT_STAT_TIME(which) {                \
+    struct timespec t = {buf->st_##which##time,       \
+                         buf->st_##which##timensec};  \
+    fake_clock_gettime(CLOCK_REALTIME, &t);           \
+    buf->st_##which##time = t.tv_sec;                 \
+    buf->st_##which##timensec = t.tv_nsec;            \
+  } while (0)
+
+static inline void fake_statbuf (struct stat *buf) {
+#ifndef st_atime
+  FAKE_STRUCT_STAT_TIME(c);
+  FAKE_STRUCT_STAT_TIME(a);
+  FAKE_STRUCT_STAT_TIME(m);
+#else
+  fake_clock_gettime(CLOCK_REALTIME, &buf->st_ctim);
+  fake_clock_gettime(CLOCK_REALTIME, &buf->st_atim);
+  fake_clock_gettime(CLOCK_REALTIME, &buf->st_mtim);
+#endif
+}
+
+static inline void fake_stat64buf (struct stat64 *buf) {
+#ifndef st_atime
+  FAKE_STRUCT_STAT_TIME(c);
+  FAKE_STRUCT_STAT_TIME(a);
+  FAKE_STRUCT_STAT_TIME(m);
+#else
+  fake_clock_gettime(CLOCK_REALTIME, &buf->st_ctim);
+  fake_clock_gettime(CLOCK_REALTIME, &buf->st_atim);
+  fake_clock_gettime(CLOCK_REALTIME, &buf->st_mtim);
+#endif
+}
+
 /* Contributed by Philipp Hachtmann in version 0.6 */
 int __xstat (int ver, const char *path, struct stat *buf)
 {
   if (NULL == real_stat)
-  {  /* dlsym() failed */
+  { /* dlsym() failed */
 #ifdef DEBUG
     (void) fprintf(stderr, "faketime problem: original stat() not found.\n");
 #endif
@@ -468,9 +514,7 @@ int __xstat (int ver, const char *path, struct stat *buf)
    {
      if (!fake_stat_disabled)
      {
-       buf->st_ctime = fake_time(&(buf->st_ctime));
-       buf->st_atime = fake_time(&(buf->st_atime));
-       buf->st_mtime = fake_time(&(buf->st_mtime));
+       fake_statbuf(buf);
      }
    }
 
@@ -499,9 +543,7 @@ int __fxstat (int ver, int fildes, struct stat *buf)
   {
     if (!fake_stat_disabled)
     {
-      buf->st_ctime = fake_time(&(buf->st_ctime));
-      buf->st_atime = fake_time(&(buf->st_atime));
-      buf->st_mtime = fake_time(&(buf->st_mtime));
+      fake_statbuf(buf);
     }
   }
   return result;
@@ -512,7 +554,7 @@ int __fxstat (int ver, int fildes, struct stat *buf)
 int __fxstatat(int ver, int fildes, const char *filename, struct stat *buf, int flag)
 {
   if (NULL == real_fstatat)
-  {  /* dlsym() failed */
+  { /* dlsym() failed */
 #ifdef DEBUG
     (void) fprintf(stderr, "faketime problem: original fstatat() not found.\n");
 #endif
@@ -528,10 +570,9 @@ int __fxstatat(int ver, int fildes, const char *filename, struct stat *buf, int
 
   if (buf != NULL)
   {
-    if (!fake_stat_disabled) {
-      buf->st_ctime = fake_time(&(buf->st_ctime));
-      buf->st_atime = fake_time(&(buf->st_atime));
-      buf->st_mtime = fake_time(&(buf->st_mtime));
+    if (!fake_stat_disabled)
+    {
+      fake_statbuf(buf);
     }
   }
   return result;
@@ -560,9 +601,7 @@ int __lxstat (int ver, const char *path, struct stat *buf)
   {
     if (!fake_stat_disabled)
     {
-      buf->st_ctime = fake_time(&(buf->st_ctime));
-      buf->st_atime = fake_time(&(buf->st_atime));
-      buf->st_mtime = fake_time(&(buf->st_mtime));
+      fake_statbuf(buf);
     }
   }
   return result;
@@ -572,7 +611,7 @@ int __lxstat (int ver, const char *path, struct stat *buf)
 int __xstat64 (int ver, const char *path, struct stat64 *buf)
 {
   if (NULL == real_stat64)
-  {  /* dlsym() failed */
+  { /* dlsym() failed */
 #ifdef DEBUG
     (void) fprintf(stderr, "faketime problem: original stat() not found.\n");
 #endif
@@ -590,9 +629,7 @@ int __xstat64 (int ver, const char *path, struct stat64 *buf)
   {
     if (!fake_stat_disabled)
     {
-      buf->st_ctime = fake_time(&(buf->st_ctime));
-      buf->st_atime = fake_time(&(buf->st_atime));
-      buf->st_mtime = fake_time(&(buf->st_mtime));
+      fake_stat64buf(buf);
     }
   }
   return result;
@@ -620,9 +657,7 @@ int __fxstat64 (int ver, int fildes, struct stat64 *buf)
   {
     if (!fake_stat_disabled)
     {
-      buf->st_ctime = fake_time(&(buf->st_ctime));
-      buf->st_atime = fake_time(&(buf->st_atime));
-      buf->st_mtime = fake_time(&(buf->st_mtime));
+      fake_stat64buf(buf);
     }
   }
   return result;
@@ -651,9 +686,7 @@ int __fxstatat64 (int ver, int fildes, const char *filename, struct stat64 *buf,
   {
     if (!fake_stat_disabled)
     {
-      buf->st_ctime = fake_time(&(buf->st_ctime));
-      buf->st_atime = fake_time(&(buf->st_atime));
-      buf->st_mtime = fake_time(&(buf->st_mtime));
+      fake_stat64buf(buf);
     }
   }
   return result;
@@ -682,9 +715,7 @@ int __lxstat64 (int ver, const char *path, struct stat64 *buf)
   {
     if (!fake_stat_disabled)
     {
-      buf->st_ctime = fake_time(&(buf->st_ctime));
-      buf->st_atime = fake_time(&(buf->st_atime));
-      buf->st_mtime = fake_time(&(buf->st_mtime));
+      fake_stat64buf(buf);
     }
   }
   return result;
@@ -785,11 +816,11 @@ unsigned int sleep(unsigned int seconds)
 {
   if (user_rate_set && !dont_fake)
   {
-    if (real_nanosleep == NULL) 
+    if (real_nanosleep == NULL)
     {
       /* fall back to sleep */
       unsigned int ret;
-      if (real_sleep == NULL) 
+      if (real_sleep == NULL)
       {
         return 0;
       }
@@ -847,7 +878,7 @@ unsigned int alarm(unsigned int seconds)
  * Faked ppoll()
  */
 int ppoll(struct pollfd *fds, nfds_t nfds,
-	  const struct timespec *timeout_ts, const sigset_t *sigmask)
+    const struct timespec *timeout_ts, const sigset_t *sigmask)
 {
   struct timespec real_timeout, *real_timeout_pt;
   int ret;
@@ -868,7 +899,8 @@ int ppoll(struct pollfd *fds, nfds_t nfds,
       /* cast away constness */
       real_timeout_pt = (struct timespec *)timeout_ts;
     }
-  } else
+  }
+  else
   {
     real_timeout_pt = NULL;
   }
@@ -934,13 +966,13 @@ timer_settime_common(timer_t_or_int timerid, int flags,
   {
     /* set it_value */
     if ((new_value->it_value.tv_sec != 0) ||
-        (new_value->it_value.tv_nsec != 0)) 
+        (new_value->it_value.tv_nsec != 0))
     {
-      if (flags & TIMER_ABSTIME) 
+      if (flags & TIMER_ABSTIME)
       {
         struct timespec tdiff, timeadj;
         timespecsub(&new_value->it_value, &user_faked_time_timespec, &timeadj);
-        if (user_rate_set) 
+        if (user_rate_set)
         {
           timespecmul(&timeadj, 1.0/user_rate, &tdiff);
         }
@@ -953,7 +985,7 @@ timer_settime_common(timer_t_or_int timerid, int flags,
       }
       else
       {
-        if (user_rate_set) 
+        if (user_rate_set)
         {
           timespecmul(&new_value->it_value, 1.0/user_rate, &new_real.it_value);
         }
@@ -985,7 +1017,7 @@ timer_settime_common(timer_t_or_int timerid, int flags,
       DONT_FAKE_TIME(result = (*real_timer_settime_22)(timerid.int_member, flags,
                     new_real_pt, old_value));
       break;
-     case FT_COMPAT_GLIBC_2_3_3:
+    case FT_COMPAT_GLIBC_2_3_3:
        DONT_FAKE_TIME(result = (*real_timer_settime_233)(timerid.timer_t_member,
                     flags, new_real_pt, old_value));
        break;
@@ -1280,14 +1312,16 @@ static void parse_ft_string(const char *user_faked_time)
   {
 
     default:  /* Try and interpret this as a specified time */
-      ft_mode = FT_FREEZE;
+      if (ft_mode != FT_NOOP) ft_mode = FT_FREEZE;
       user_faked_time_tm.tm_isdst = -1;
       if (NULL != strptime(user_faked_time, user_faked_time_fmt, &user_faked_time_tm))
       {
         user_faked_time_timespec.tv_sec = mktime(&user_faked_time_tm);
         user_faked_time_timespec.tv_nsec = 0;
         user_faked_time_set = true;
-      } else {
+      }
+      else
+      {
         perror("Failed to parse FAKETIME timestamp");
         exit(EXIT_FAILURE);
       }
@@ -1295,7 +1329,7 @@ static void parse_ft_string(const char *user_faked_time)
 
     case '+':
     case '-': /* User-specified offset */
-      ft_mode = FT_START_AT;
+      if (ft_mode != FT_NOOP) ft_mode = FT_START_AT;
       /* fractional time offsets contributed by Karl Chen in v0.8 */
       double frac_offset = atof(user_faked_time);
 
@@ -1356,6 +1390,12 @@ void __attribute__ ((constructor)) ftpl_init(void)
 {
   char *tmp_env;
 
+#ifdef __APPLE__
+  const char *progname = getprogname();
+#else
+  const char *progname = __progname;
+#endif
+
   /* Look up all real_* functions. NULL will mark missing ones. */
   real_stat =               dlsym(RTLD_NEXT, "__xstat");
   real_fstat =              dlsym(RTLD_NEXT, "__fxstat");
@@ -1376,6 +1416,11 @@ void __attribute__ ((constructor)) ftpl_init(void)
   real_poll =               dlsym(RTLD_NEXT, "poll");
   real_ppoll =              dlsym(RTLD_NEXT, "ppoll");
 #endif
+#ifdef FAKE_INTERNAL_CALLS
+  real___ftime =              dlsym(RTLD_NEXT, "__ftime");
+  real___gettimeofday =       dlsym(RTLD_NEXT, "__gettimeofday");
+  real___clock_gettime  =     dlsym(RTLD_NEXT, "__clock_gettime");
+#endif
 #ifdef __APPLE__
   real_clock_get_time =     dlsym(RTLD_NEXT, "clock_get_time");
   real_clock_gettime  =     apple_clock_gettime;
@@ -1384,8 +1429,16 @@ void __attribute__ ((constructor)) ftpl_init(void)
 #ifdef FAKE_TIMERS
   real_timer_settime_22 =   dlvsym(RTLD_NEXT, "timer_settime","GLIBC_2.2");
   real_timer_settime_233 =  dlvsym(RTLD_NEXT, "timer_settime","GLIBC_2.3.3");
+  if (NULL == real_timer_settime_233)
+  {
+    real_timer_settime_233 =  dlsym(RTLD_NEXT, "timer_settime");
+  }
   real_timer_gettime_22 =   dlvsym(RTLD_NEXT, "timer_gettime","GLIBC_2.2");
   real_timer_gettime_233 =  dlvsym(RTLD_NEXT, "timer_gettime","GLIBC_2.3.3");
+  if (NULL == real_timer_gettime_233)
+  {
+    real_timer_gettime_233 =  dlsym(RTLD_NEXT, "timer_gettime");
+  }
 #endif
 #endif
 
@@ -1399,6 +1452,53 @@ void __attribute__ ((constructor)) ftpl_init(void)
 
   /* Check whether we actually should be faking the returned timestamp. */
 
+  /* We can prevent faking time for specified commands */
+  if ((tmp_env = getenv("FAKETIME_SKIP_CMDS")) != NULL)
+  {
+    char *skip_cmd, *saveptr;
+    skip_cmd = strtok_r(tmp_env, ",", &saveptr);
+    while (skip_cmd != NULL)
+    {
+      if (0 == strcmp(progname, skip_cmd))
+      {
+        ft_mode = FT_NOOP;
+        dont_fake = true;
+        break;
+      }
+      skip_cmd = strtok_r(NULL, ",", &saveptr);
+    }
+  }
+
+  /* We can limit faking time to specified commands */
+  if ((tmp_env = getenv("FAKETIME_ONLY_CMDS")) != NULL)
+  {
+    char *only_cmd, *saveptr;
+    bool cmd_matched = false;
+
+    if (getenv("FAKETIME_SKIP_CMDS") != NULL)
+    {
+      fprintf(stderr, "Error: Both FAKETIME_SKIP_CMDS and FAKETIME_ONLY_CMDS can't be set.\n");
+      exit(EXIT_FAILURE);
+    }
+
+    only_cmd = strtok_r(tmp_env, ",", &saveptr);
+    while (only_cmd != NULL)
+    {
+      if (0 == strcmp(progname, only_cmd))
+      {
+        cmd_matched = true;
+        break;
+      }
+      only_cmd = strtok_r(NULL, ",", &saveptr);
+    }
+
+    if (!cmd_matched)
+    {
+      ft_mode = FT_NOOP;
+      dont_fake = true;
+    }
+  }
+
   if ((tmp_env = getenv("FAKETIME_START_AFTER_SECONDS")) != NULL)
   {
     ft_start_after_secs = atol(tmp_env);
@@ -1431,17 +1531,17 @@ void __attribute__ ((constructor)) ftpl_init(void)
     }
     if ((tmp_env = getenv("FAKETIME_SPAWN_NUMCALLS")) != NULL)
     {
-  	  ft_spawn_ncalls = atol(tmp_env);
+      ft_spawn_ncalls = atol(tmp_env);
     }
   }
 
   if ((tmp_env = getenv("FAKETIME_SAVE_FILE")) != NULL)
   {
     if (-1 == (outfile = open(tmp_env, O_RDWR | O_APPEND | O_CLOEXEC | O_CREAT,
-			                  S_IWUSR | S_IRUSR)))
+                              S_IWUSR | S_IRUSR)))
     {
-  	  perror("Opening file for saving timestamps failed");
-	  exit(EXIT_FAILURE);
+      perror("Opening file for saving timestamps failed");
+      exit(EXIT_FAILURE);
     }
   }
 
@@ -1460,7 +1560,7 @@ void __attribute__ ((constructor)) ftpl_init(void)
     if (sizeof(stss[0]) > (infile_size = sb.st_size))
     {
       printf("There are no timestamps in the provided file to load timestamps from");
-	  exit(EXIT_FAILURE);
+      exit(EXIT_FAILURE);
     }
 
     if ((infile_size % sizeof(stss[0])) != 0)
@@ -1473,7 +1573,7 @@ void __attribute__ ((constructor)) ftpl_init(void)
     if (stss == MAP_FAILED)
     {
       perror("Mapping file for loading timestamps failed");
-	  exit(EXIT_FAILURE);
+      exit(EXIT_FAILURE);
     }
     infile_set = true;
   }
@@ -1492,13 +1592,13 @@ void __attribute__ ((constructor)) ftpl_init(void)
   {
     if (sem_wait(shared_sem) == -1)
     {
-  	  perror("sem_wait");
+      perror("sem_wait");
       exit(1);
     }
     if (ft_shared->start_time.real.tv_nsec == -1)
     {
       /* set up global start time */
-	  system_time_from_system(&ftpl_starttime);
+      system_time_from_system(&ftpl_starttime);
       ft_shared->start_time = ftpl_starttime;
     }
     else
@@ -1508,7 +1608,7 @@ void __attribute__ ((constructor)) ftpl_init(void)
     }
     if (sem_post(shared_sem) == -1)
     {
-	  perror("sem_post");
+      perror("sem_post");
       exit(1);
     }
   }
@@ -1542,7 +1642,9 @@ static void remove_trailing_eols(char *line)
    */
 # define is_eolchar(c) ((c) == '\n' || (c) == '\r')
   while (endp > line && is_eolchar(endp[-1]))
-	*--endp = '\0';
+  {
+    *--endp = '\0';
+  }
 }
 
 
@@ -1594,12 +1696,12 @@ int fake_clock_gettime(clockid_t clk_id, struct timespec *tp)
     {
       case CLOCK_REALTIME:
         timespecsub(tp, &ftpl_starttime.real, &tmp_ts);
-    	break;
+        break;
       case CLOCK_MONOTONIC:
         timespecsub(tp, &ftpl_starttime.mon, &tmp_ts);
         break;
       case CLOCK_MONOTONIC_RAW:
-    	timespecsub(tp, &ftpl_starttime.mon_raw, &tmp_ts);
+        timespecsub(tp, &ftpl_starttime.mon_raw, &tmp_ts);
         break;
       default:
         printf("Invalid clock_id for clock_gettime: %d", clk_id);
@@ -1609,12 +1711,12 @@ int fake_clock_gettime(clockid_t clk_id, struct timespec *tp)
     if (limited_faking)
     {
       /* Check whether we actually should be faking the returned timestamp. */
-  	  /* fprintf(stderr, "(libfaketime limits -> runtime: %lu, callcounter: %lu\n", (*time_tptr - ftpl_starttime), callcounter); */
+      /* fprintf(stderr, "(libfaketime limits -> runtime: %lu, callcounter: %lu\n", (*time_tptr - ftpl_starttime), callcounter); */
       if ((ft_start_after_secs != -1)   && (tmp_ts.tv_sec < ft_start_after_secs)) return 0;
-  	  if ((ft_stop_after_secs != -1)    && (tmp_ts.tv_sec >= ft_stop_after_secs)) return 0;
+      if ((ft_stop_after_secs != -1)    && (tmp_ts.tv_sec >= ft_stop_after_secs)) return 0;
       if ((ft_start_after_ncalls != -1) && (callcounter < ft_start_after_ncalls)) return 0;
-  	  if ((ft_stop_after_ncalls != -1)  && (callcounter >= ft_stop_after_ncalls)) return 0;
-   	  /* fprintf(stderr, "(libfaketime limits -> runtime: %lu, callcounter: %lu continues\n", (*time_tptr - ftpl_starttime), callcounter); */
+      if ((ft_stop_after_ncalls != -1)  && (callcounter >= ft_stop_after_ncalls)) return 0;
+      /* fprintf(stderr, "(libfaketime limits -> runtime: %lu, callcounter: %lu continues\n", (*time_tptr - ftpl_starttime), callcounter); */
     }
 
     if (spawnsupport)
@@ -1649,21 +1751,28 @@ int fake_clock_gettime(clockid_t clk_id, struct timespec *tp)
 
   if (cache_expired == 1)
   {
-    static char user_faked_time[BUFFERLEN]; /* changed to static for caching in v0.6 */
-    char filename[BUFSIZ], line[BUFFERLEN];
-	FILE *faketimerc;
-
     last_data_fetch = tp->tv_sec;
     /* Can be enabled for testing ...
       fprintf(stderr, "***************++ Cache expired ++**************\n");
     */
 
-    /* initialize with default */
-    snprintf(user_faked_time, BUFFERLEN, "+0");
-
     /* fake time supplied as environment variable? */
     if (parse_config_file)
     {
+      static char user_faked_time[BUFFERLEN]; /* changed to static for caching in v0.6 */
+      char filename[BUFSIZ];
+      FILE *faketimerc;
+      /* initialize with default or env. variable */
+      char *tmp_env;
+      if (NULL != (tmp_env = getenv("FAKETIME")))
+      {
+        strncpy(user_faked_time, tmp_env, BUFFERLEN);
+      }
+      else
+      {
+        snprintf(user_faked_time, BUFFERLEN, "+0");
+      }
+
       /* check whether there's a .faketimerc in the user's home directory, or
        * a system-wide /etc/faketimerc present.
        * The /etc/faketimerc handling has been contributed by David Burley,
@@ -1672,6 +1781,7 @@ int fake_clock_gettime(clockid_t clk_id, struct timespec *tp)
       if ((faketimerc = fopen(filename, "rt")) != NULL ||
           (faketimerc = fopen("/etc/faketimerc", "rt")) != NULL)
       {
+        char line[BUFFERLEN];
         while(fgets(line, BUFFERLEN, faketimerc) != NULL)
         {
           if ((strlen(line) > 1) && (line[0] != ' ') &&
@@ -1693,7 +1803,7 @@ int fake_clock_gettime(clockid_t clk_id, struct timespec *tp)
   {
     if (load_time(tp))
     {
-  	return 0;
+      return 0;
     }
   }
 
@@ -1710,36 +1820,36 @@ int fake_clock_gettime(clockid_t clk_id, struct timespec *tp)
     case FT_START_AT: /* User-specified offset */
       if (user_per_tick_inc_set)
       {
-    	/* increment time with every time() call*/
+        /* increment time with every time() call*/
         next_time(tp, &user_per_tick_inc);
       }
       else
       {
         /* Speed-up / slow-down contributed by Karl Chen in v0.8 */
-   	    struct timespec tdiff, timeadj;
+        struct timespec tdiff, timeadj;
         switch (clk_id)
         {
           case CLOCK_REALTIME:
-   	        timespecsub(tp, &ftpl_starttime.real, &tdiff);
-       	    break;
-       	  case CLOCK_MONOTONIC:
-            timespecsub(tp, &ftpl_starttime.mon, &tdiff);
-      	    break;
-          case CLOCK_MONOTONIC_RAW:
-      	    timespecsub(tp, &ftpl_starttime.mon_raw, &tdiff);
+            timespecsub(tp, &ftpl_starttime.real, &tdiff);
             break;
-      	  default:
-      	    printf("Invalid clock_id for clock_gettime: %d", clk_id);
-      	    exit(EXIT_FAILURE);
+          case CLOCK_MONOTONIC:
+            timespecsub(tp, &ftpl_starttime.mon, &tdiff);
+            break;
+          case CLOCK_MONOTONIC_RAW:
+            timespecsub(tp, &ftpl_starttime.mon_raw, &tdiff);
+            break;
+          default:
+            printf("Invalid clock_id for clock_gettime: %d", clk_id);
+            exit(EXIT_FAILURE);
         } // end of switch (clk_id)
         if (user_rate_set)
         {
-  	      timespecmul(&tdiff, user_rate, &timeadj);
+          timespecmul(&tdiff, user_rate, &timeadj);
         }
         else
         {
           timeadj = tdiff;
-	    }
+        }
         timespecadd(&user_faked_time_timespec, &timeadj, tp);
       }
       break;
@@ -1766,22 +1876,6 @@ time_t fake_time(time_t *time_tptr)
   return *time_tptr;
 }
 
-/* ftime() is faked otherwise since v0.9.5
-int fake_ftime(struct timeb *tp)
-{
-  struct timespec ts;
-  int ret;
-  ts.tv_sec = tp->time;
-  ts.tv_nsec =tp->millitm * 1000000 + ftpl_starttime.real.tv_nsec % 1000000;
-
-  ret = fake_clock_gettime(CLOCK_REALTIME, &ts);
-  tp->time = ts.tv_sec;
-  tp->millitm =ts.tv_nsec / 1000000;
-
-  return ret;
-}
-*/
-
 int fake_gettimeofday(struct timeval *tv)
 {
   struct timespec ts;
@@ -1808,7 +1902,8 @@ int fake_gettimeofday(struct timeval *tv)
  * clock_gettime implementation for __APPLE__
  * @note It always behave like being called with CLOCK_REALTIME.
  */
-static int apple_clock_gettime(clockid_t clk_id, struct timespec *tp) {
+static int apple_clock_gettime(clockid_t clk_id, struct timespec *tp)
+{
   int result;
   mach_timespec_t cur_timeclockid_t;
   (void) clk_id; /* unused */
@@ -1857,32 +1952,122 @@ int clock_get_time(clock_serv_t clock_serv, mach_timespec_t *cur_timeclockid_t)
  *      =======================================================================
  */
 
-/*
- * The following __interceptions cause serious issues in Mac OS X 10.7 (and higher) and are therefore #ifndef'ed
- */
-#ifndef __APPLE__
-/* Added in v0.7 as suggested by Jamie Cameron, Google */
 #ifdef FAKE_INTERNAL_CALLS
 int __gettimeofday(struct timeval *tv, void *tz)
 {
-    return gettimeofday(tv, tz);
+  int result;
+
+  /* sanity check */
+  if (tv == NULL)
+  {
+    return -1;
+  }
+
+  /* Check whether we've got a pointer to the real ftime() function yet */
+  if (NULL == real___gettimeofday)
+  {  /* dlsym() failed */
+#ifdef DEBUG
+    (void) fprintf(stderr, "faketime problem: original __gettimeofday() not found.\n");
+#endif
+    return -1; /* propagate error to caller */
+  }
+
+  /* initialize our result with the real current time */
+  DONT_FAKE_TIME(result = (*real___gettimeofday)(tv, tz));
+  if (result == -1) return result; /* original function failed */
+
+  /* pass the real current time to our faking version, overwriting it */
+  result = fake_gettimeofday(tv);
+
+  /* return the result to the caller */
+  return result;
 }
 
 int __clock_gettime(clockid_t clk_id, struct timespec *tp)
 {
-    return clock_gettime(clk_id, tp);
-}
+  int result;
 
-int __ftime(struct timeb *tp)
-{
-    return ftime(tp);
+  /* sanity check */
+  if (tp == NULL)
+  {
+    return -1;
+  }
+
+  if (NULL == real___clock_gettime)
+  {  /* dlsym() failed */
+#ifdef DEBUG
+    (void) fprintf(stderr, "faketime problem: original __clock_gettime() not found.\n");
+#endif
+    return -1; /* propagate error to caller */
+  }
+
+  /* initialize our result with the real current time */
+  DONT_FAKE_TIME(result = (*real___clock_gettime)(clk_id, tp));
+  if (result == -1) return result; /* original function failed */
+
+  /* pass the real current time to our faking version, overwriting it */
+  result = fake_clock_gettime(clk_id, tp);
+
+  /* return the result to the caller */
+  return result;
 }
 
 time_t __time(time_t *time_tptr)
 {
-    return time(time_tptr);
+  struct timespec tp;
+  time_t result;
+
+  DONT_FAKE_TIME(result = (*real_clock_gettime)(CLOCK_REALTIME, &tp));
+  if (result == -1) return -1;
+
+  /* pass the real current time to our faking version, overwriting it */
+  (void)fake_clock_gettime(CLOCK_REALTIME, &tp);
+
+  if (time_tptr != NULL)
+  {
+    *time_tptr = tp.tv_sec;
+  }
+  return tp.tv_sec;
 }
+
+int __ftime(struct timeb *tb)
+{
+  struct timespec tp;
+  int result;
+
+  /* sanity check */
+  if (tb == NULL)
+    return 0;               /* ftime() always returns 0, see manpage */
+
+  /* Check whether we've got a pointer to the real ftime() function yet */
+  if (NULL == real___ftime)
+  {  /* dlsym() failed */
+#ifdef DEBUG
+    (void) fprintf(stderr, "faketime problem: original ftime() not found.\n");
 #endif
+    return 0; /* propagate error to caller */
+  }
+
+  /* initialize our TZ result with the real current time */
+  DONT_FAKE_TIME(result = (*real___ftime)(tb));
+  if (result == -1)
+  {
+    return result;
+  }
+
+  DONT_FAKE_TIME(result = (*real_clock_gettime)(CLOCK_REALTIME, &tp));
+  if (result == -1) return -1;
+
+  /* pass the real current time to our faking version, overwriting it */
+  (void)fake_clock_gettime(CLOCK_REALTIME, &tp);
+
+  tb->time = tp.tv_sec;
+  tb->millitm = tp.tv_nsec / 1000000;
+
+  /* return the result to the caller */
+  return result; /* will always be 0 (see manpage) */
+}
+
 #endif
 
 /*
